from transformers import pipeline
from crewai.tools import BaseTool
from typing import Type
from pydantic import PrivateAttr, BaseModel


class FinBERTSentimentAnalysisToolSchema(BaseModel):
    text: str


class FinBERTSentimentAnalysisTool(BaseTool):
    """A tool for analyzing sentiment using the FinBERT model."""
    name: str = "finbert_sentiment_analysis_tool"
    description: str = "A tool for analyzing sentiment using the FinBERT model."
    args_schema: Type[BaseModel] = FinBERTSentimentAnalysisToolSchema
    _classifier: pipeline = PrivateAttr()

    def __init__(self, **kwargs):
        """
        Initialize the FinBERT model as a private attribute to avoid Pydantic validation issues.
        """
        super().__init__(**kwargs)
        self._classifier = pipeline("sentiment-analysis", model="yiyanghkust/finbert-tone")

    def _run(self, text: str):
        """
        Analyzes sentiment of a given text.

        Args:
            text (str): The text to analyze.

        Returns:
            dict: A dictionary with sentiment analysis results.
        """
        try:
            result = self._classifier(text)
            sentiment = result[0]['label']
            score = round(result[0]['score'], 2)

            return {"sentiment": sentiment, "confidence": score}

        except Exception as e:
            return {"error": f"Error performing sentiment analysis: {str(e)}"}


if __name__ == "__main__":
    # Simple local test for FinBERTSentimentAnalysisTool
    tool_instance = FinBERTSentimentAnalysisTool()
    sample_text = "I love this product, it is absolutely fantastic!"
    response = tool_instance.run(text=sample_text)
    print("Sentiment Tool Response:", response)



import os
import re
from typing import Any, Optional, Type

import requests
from bs4 import BeautifulSoup
from crewai.tools import BaseTool
from pydantic import BaseModel, Field


class FixedScrapeWebsiteToolSchema(BaseModel):
    """Input for ScrapeWebsiteTool."""
    pass


class ScrapeWebsiteToolSchema(FixedScrapeWebsiteToolSchema):
    """Input for ScrapeWebsiteTool."""
    website_url: str = Field(..., description="Mandatory website url to read the file")


class ScrapeWebsiteTool(BaseTool):
    name: str = "Read website content"
    description: str = "A tool that can be used to read a website content."
    args_schema: Type[BaseModel] = ScrapeWebsiteToolSchema
    website_url: Optional[str] = None
    cookies: Optional[dict] = None
    headers: Optional[dict] = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Language": "en-US,en;q=0.9",
        "Referer": "https://www.google.com/",
        "Connection": "keep-alive",
        "Upgrade-Insecure-Requests": "1",
    }

    def __init__(
        self,
        website_url: Optional[str] = None,
        cookies: Optional[dict] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        if website_url is not None:
            self.website_url = website_url
            self.description = f"A tool that can be used to read {website_url}'s content."
            self.args_schema = FixedScrapeWebsiteToolSchema
            self._generate_description()
            if cookies is not None:
                self.cookies = {cookies["name"]: os.getenv(cookies["value"])}

    def _run(
        self,
        **kwargs: Any,
    ) -> Any:
        website_url = kwargs.get("website_url", self.website_url)
        page = requests.get(
            website_url,
            timeout=15,
            headers=self.headers,
            cookies=self.cookies if self.cookies else {},
        )

        page.encoding = page.apparent_encoding
        parsed = BeautifulSoup(page.text, "html.parser")

        text = parsed.get_text(" ")
        text = re.sub("[ \t]+", " ", text)
        text = re.sub("\\s+\n\\s+", "\n", text)
        return text


if __name__ == "__main__":
    # Test the tool
    tool = ScrapeWebsiteTool()
    result = tool.run("https://www.example.com")
    print(result) 


from .coingecko_quote_tool.coingecko_quote_tool import CoingeckoUniversalQuoteTool
from .sentiment_analysis_tool.sentiment_analysis_tool import FinBERTSentimentAnalysisTool

# TODO: After verification and testing, import crewai tools below from local folders as traia tools.
from crewai_tools.tools.scrape_website_tool.scrape_website_tool import ScrapeWebsiteTool
from crewai_tools.tools.serply_api_tool.serply_news_search_tool import SerplyNewsSearchTool
from crewai_tools.tools.serper_dev_tool.serper_dev_tool import SerperDevTool
from crewai_tools.tools.json_search_tool.json_search_tool import JSONSearchTool
from crewai_tools.tools.rag.rag_tool import RagTool

__all__ = [
    'CoingeckoUniversalQuoteTool',
    'FinBERTSentimentAnalysisTool',
    'ScrapeWebsiteTool',
    'SerplyNewsSearchTool',
    'SerperDevTool',
    'JSONSearchTool',
    'RagTool'
]